@@ -4,18 +4,13 @@ import html.parser
 import pprint
 import unittest
 
-from test import support
-
 
 class EventCollector(html.parser.HTMLParser):
 
-    def __init__(self, *args, autocdata=False, **kw):
-        self.autocdata = autocdata
+    def __init__(self, *args, **kw):
         self.events = []
         self.append = self.events.append
         html.parser.HTMLParser.__init__(self, *args, **kw)
-        if autocdata:
-            self._set_support_cdata(False)
 
     def get_events(self):
         # Normalize the list of events so that buffer artefacts don't
@@ -36,16 +31,12 @@ class EventCollector(html.parser.HTMLParser):
 
     def handle_starttag(self, tag, attrs):
         self.append(("starttag", tag, attrs))
-        if self.autocdata and tag == 'svg':
-            self._set_support_cdata(True)
 
     def handle_startendtag(self, tag, attrs):
         self.append(("startendtag", tag, attrs))
 
     def handle_endtag(self, tag):
         self.append(("endtag", tag))
-        if self.autocdata and tag == 'svg':
-            self._set_support_cdata(False)
 
     # all other markup
 
@@ -87,13 +78,6 @@ class EventCollectorCharrefs(EventCollector):
         self.fail('This should never be called with convert_charrefs=True')
 
 
-# The normal event collector normalizes the events in get_events,
-# so we override it to return the original list of events.
-class EventCollectorNoNormalize(EventCollector):
-    def get_events(self):
-        return self.events
-
-
 class TestCaseBase(unittest.TestCase):
 
     def get_collector(self):
@@ -278,7 +262,8 @@ text
             ("starttag", "foo:bar", [("one", "1"), ("two", "2")]),
             ("starttag_text", s)])
 
-    @support.subTests('content', [
+    def test_cdata_content(self):
+        contents = [
             '<!-- not a comment --> &not-an-entity-ref;',
             "<not a='start tag'>",
             '<a href="" /> <p> <span></span>',
@@ -291,223 +276,60 @@ text
              'src="http://www.example.org/r=\'+new '
              'Date().getTime()+\'"><\\/s\'+\'cript>\');\n//]]>'),
             '\n<!-- //\nvar foo = 3.14;\n// -->\n',
+            'foo = "</sty" + "le>";',
             '<!-- \u2603 -->',
-            'foo = "</ script>"',
-            'foo = "</scripture>"',
-            'foo = "</script\v>"',
-            'foo = "</script\xa0>"',
-            'foo = "</ſcript>"',
-            'foo = "</scrıpt>"',
-        ])
-    def test_script_content(self, content):
-        s = f'<script>{content}</script>'
-        self._run_check(s, [("starttag", "script", []),
-                            ("data", content),
-                            ("endtag", "script")])
+            # these two should be invalid according to the HTML 5 spec,
+            # section 8.1.2.2
+            #'foo = </\nscript>',
+            #'foo = </ script>',
+        ]
+        elements = ['script', 'style', 'SCRIPT', 'STYLE', 'Script', 'Style']
+        for content in contents:
+            for element in elements:
+                element_lower = element.lower()
+                s = '<{element}>{content}</{element}>'.format(element=element,
+                                                               content=content)
+                self._run_check(s, [("starttag", element_lower, []),
+                                    ("data", content),
+                                    ("endtag", element_lower)])
 
-    @support.subTests('content', [
-            'a::before { content: "<!-- not a comment -->"; }',
-            'a::before { content: "&not-an-entity-ref;"; }',
-            'a::before { content: "<not a=\'start tag\'>"; }',
-            'a::before { content: "\u2603"; }',
-            'a::before { content: "< /style>"; }',
-            'a::before { content: "</ style>"; }',
-            'a::before { content: "</styled>"; }',
-            'a::before { content: "</style\v>"; }',
-            'a::before { content: "</style\xa0>"; }',
-            'a::before { content: "</ſtyle>"; }',
-        ])
-    def test_style_content(self, content):
-        s = f'<style>{content}</style>'
-        self._run_check(s, [("starttag", "style", []),
-                            ("data", content),
-                            ("endtag", "style")])
-
-    @support.subTests('content', [
-            '<!-- not a comment -->',
-            "<not a='start tag'>",
-            '<![CDATA[not a cdata]]>',
-            '<!not a bogus comment>',
-            '</not a bogus comment>',
-            '\u2603',
-            '< /title>',
-            '</ title>',
-            '</titled>',
-            '</title\v>',
-            '</title\xa0>',
-            '</tıtle>',
-        ])
-    def test_title_content(self, content):
-        source = f"<title>{content}</title>"
-        self._run_check(source, [
-            ("starttag", "title", []),
-            ("data", content),
-            ("endtag", "title"),
-        ])
-
-    @support.subTests('content', [
-            '<!-- not a comment -->',
-            "<not a='start tag'>",
-            '<![CDATA[not a cdata]]>',
-            '<!not a bogus comment>',
-            '</not a bogus comment>',
-            '\u2603',
-            '< /textarea>',
-            '</ textarea>',
-            '</textareable>',
-            '</textarea\v>',
-            '</textarea\xa0>',
-        ])
-    def test_textarea_content(self, content):
-        source = f"<textarea>{content}</textarea>"
-        self._run_check(source, [
-            ("starttag", "textarea", []),
-            ("data", content),
-            ("endtag", "textarea"),
-        ])
-
-    @support.subTests('endtag', ['script', 'SCRIPT', 'script ', 'script\n',
-                                 'script/', 'script foo=bar', 'script foo=">"'])
-    def test_script_closing_tag(self, endtag):
+    def test_cdata_with_closing_tags(self):
         # see issue #13358
         # make sure that HTMLParser calls handle_data only once for each CDATA.
+        # The normal event collector normalizes  the events in get_events,
+        # so we override it to return the original list of events.
+        class Collector(EventCollector):
+            def get_events(self):
+                return self.events
+
         content = """<!-- not a comment --> &not-an-entity-ref;
                   <a href="" /> </p><p> <span></span></style>
                   '</script' + '>'"""
-        s = f'<ScrIPt>{content}</{endtag}>'
-        self._run_check(s, [("starttag", "script", []),
-                            ("data", content),
-                            ("endtag", "script")],
-                        collector=EventCollectorNoNormalize(convert_charrefs=False))
-
-    @support.subTests('endtag', ['style', 'STYLE', 'style ', 'style\n',
-                                 'style/', 'style foo=bar', 'style foo=">"'])
-    def test_style_closing_tag(self, endtag):
-        content = """
-            b::before { content: "<!-- not a comment -->"; }
-            p::before { content: "&not-an-entity-ref;"; }
-            a::before { content: "<i>"; }
-            a::after { content: "</i>"; }
-            """
-        s = f'<StyLE>{content}</{endtag}>'
-        self._run_check(s, [("starttag", "style", []),
-                            ("data", content),
-                            ("endtag", "style")],
-                        collector=EventCollectorNoNormalize(convert_charrefs=False))
-
-    @support.subTests('endtag', ['title', 'TITLE', 'title ', 'title\n',
-                                 'title/', 'title foo=bar', 'title foo=">"'])
-    def test_title_closing_tag(self, endtag):
-        content = "<!-- not a comment --><i>Egg &amp; Spam</i>"
-        s = f'<TitLe>{content}</{endtag}>'
-        self._run_check(s, [("starttag", "title", []),
-                            ('data', '<!-- not a comment --><i>Egg & Spam</i>'),
-                            ("endtag", "title")],
-                        collector=EventCollectorNoNormalize(convert_charrefs=True))
-        self._run_check(s, [("starttag", "title", []),
-                            ('data', '<!-- not a comment --><i>Egg '),
-                            ('entityref', 'amp'),
-                            ('data', ' Spam</i>'),
-                            ("endtag", "title")],
-                        collector=EventCollectorNoNormalize(convert_charrefs=False))
-
-    @support.subTests('endtag', ['textarea', 'TEXTAREA', 'textarea ', 'textarea\n',
-                                 'textarea/', 'textarea foo=bar', 'textarea foo=">"'])
-    def test_textarea_closing_tag(self, endtag):
-        content = "<!-- not a comment --><i>Egg &amp; Spam</i>"
-        s = f'<TexTarEa>{content}</{endtag}>'
-        self._run_check(s, [("starttag", "textarea", []),
-                            ('data', '<!-- not a comment --><i>Egg & Spam</i>'),
-                            ("endtag", "textarea")],
-                        collector=EventCollectorNoNormalize(convert_charrefs=True))
-        self._run_check(s, [("starttag", "textarea", []),
-                            ('data', '<!-- not a comment --><i>Egg '),
-                            ('entityref', 'amp'),
-                            ('data', ' Spam</i>'),
-                            ("endtag", "textarea")],
-                        collector=EventCollectorNoNormalize(convert_charrefs=False))
-
-    @support.subTests('tail,end', [
-        ('', False),
-        ('<', False),
-        ('</', False),
-        ('</s', False),
-        ('</script', False),
-        ('</script ', True),
-        ('</script foo=bar', True),
-        ('</script foo=">', True),
-    ])
-    def test_eof_in_script(self, tail, end):
-        content = "a = 123"
-        s = f'<ScrIPt>{content}{tail}'
-        self._run_check(s, [("starttag", "script", []),
-                            ("data", content if end else content + tail)],
-                        collector=EventCollectorNoNormalize(convert_charrefs=False))
-
-    @support.subTests('tail,end', [
-        ('', False),
-        ('<', False),
-        ('</', False),
-        ('</t', False),
-        ('</title', False),
-        ('</title ', True),
-        ('</title foo=bar', True),
-        ('</title foo=">', True),
-    ])
-    def test_eof_in_title(self, tail, end):
-        s = f'<TitLe>Egg &amp; Spam{tail}'
-        self._run_check(s, [("starttag", "title", []),
-                            ("data", "Egg & Spam" + ('' if end else tail))],
-                        collector=EventCollectorNoNormalize(convert_charrefs=True))
-        self._run_check(s, [("starttag", "title", []),
-                            ('data', 'Egg '),
-                            ('entityref', 'amp'),
-                            ('data', ' Spam' + ('' if end else tail))],
-                        collector=EventCollectorNoNormalize(convert_charrefs=False))
+        for element in [' script', 'script ', ' script ',
+                        '\nscript', 'script\n', '\nscript\n']:
+            element_lower = element.lower().strip()
+            s = '<script>{content}</{element}>'.format(element=element,
+                                                       content=content)
+            self._run_check(s, [("starttag", element_lower, []),
+                                ("data", content),
+                                ("endtag", element_lower)],
+                            collector=Collector(convert_charrefs=False))
 
     def test_comments(self):
         html = ("<!-- I'm a valid comment -->"
                 '<!--me too!-->'
                 '<!------>'
-                '<!----->'
                 '<!---->'
-                # abrupt-closing-of-empty-comment
-                '<!--->'
-                '<!-->'
                 '<!----I have many hyphens---->'
                 '<!-- I have a > in the middle -->'
-                '<!-- and I have -- in the middle! -->'
-                '<!--incorrectly-closed-comment--!>'
-                '<!----!>'
-                '<!----!-->'
-                '<!---- >-->'
-                '<!---!>-->'
-                '<!--!>-->'
-                # nested-comment
-                '<!-- <!-- nested --> -->'
-                '<!--<!-->'
-                '<!--<!--!>'
-        )
+                '<!-- and I have -- in the middle! -->')
         expected = [('comment', " I'm a valid comment "),
                     ('comment', 'me too!'),
                     ('comment', '--'),
-                    ('comment', '-'),
-                    ('comment', ''),
-                    ('comment', ''),
                     ('comment', ''),
                     ('comment', '--I have many hyphens--'),
                     ('comment', ' I have a > in the middle '),
-                    ('comment', ' and I have -- in the middle! '),
-                    ('comment', 'incorrectly-closed-comment'),
-                    ('comment', ''),
-                    ('comment', '--!'),
-                    ('comment', '-- >'),
-                    ('comment', '-!>'),
-                    ('comment', '!>'),
-                    ('comment', ' <!-- nested '), ('data', ' -->'),
-                    ('comment', '<!'),
-                    ('comment', '<!'),
-        ]
+                    ('comment', ' and I have -- in the middle! ')]
         self._run_check(html, expected)
 
     def test_condcoms(self):
@@ -569,34 +391,28 @@ text
                             ('data', '<'),
                             ('starttag', 'bc<', [('a', None)]),
                             ('endtag', 'html'),
-                            ('data', '\n')])
+                            ('data', '\n<img src="URL>'),
+                            ('comment', '/img'),
+                            ('endtag', 'html<')])
 
     def test_starttag_junk_chars(self):
-        self._run_check("<", [('data', '<')])
-        self._run_check("<>", [('data', '<>')])
-        self._run_check("< >", [('data', '< >')])
-        self._run_check("< ", [('data', '< ')])
         self._run_check("</>", [])
-        self._run_check("<$>", [('data', '<$>')])
         self._run_check("</$>", [('comment', '$')])
         self._run_check("</", [('data', '</')])
-        self._run_check("</a", [])
-        self._run_check("</ a>", [('comment', ' a')])
-        self._run_check("</ a", [('comment', ' a')])
+        self._run_check("</a", [('data', '</a')])
         self._run_check("<a<a>", [('starttag', 'a<a', [])])
         self._run_check("</a<a>", [('endtag', 'a<a')])
-        self._run_check("<!", [('comment', '')])
-        self._run_check("<a", [])
-        self._run_check("<a foo='bar'", [])
-        self._run_check("<a foo='bar", [])
-        self._run_check("<a foo='>'", [])
-        self._run_check("<a foo='>", [])
+        self._run_check("<!", [('data', '<!')])
+        self._run_check("<a", [('data', '<a')])
+        self._run_check("<a foo='bar'", [('data', "<a foo='bar'")])
+        self._run_check("<a foo='bar", [('data', "<a foo='bar")])
+        self._run_check("<a foo='>'", [('data', "<a foo='>'")])
+        self._run_check("<a foo='>", [('data', "<a foo='>")])
         self._run_check("<a$>", [('starttag', 'a$', [])])
         self._run_check("<a$b>", [('starttag', 'a$b', [])])
         self._run_check("<a$b/>", [('startendtag', 'a$b', [])])
         self._run_check("<a$b  >", [('starttag', 'a$b', [])])
         self._run_check("<a$b  />", [('startendtag', 'a$b', [])])
-        self._run_check("</a$b>", [('endtag', 'a$b')])
 
     def test_slashes_in_starttag(self):
         self._run_check('<a foo="var"/>', [('startendtag', 'a', [('foo', 'var')])])
@@ -629,10 +445,6 @@ text
         ]
         self._run_check(html, expected)
 
-    def test_slashes_in_endtag(self):
-        self._run_check('</a/>', [('endtag', 'a')])
-        self._run_check('</a foo="var"/>', [('endtag', 'a')])
-
     def test_declaration_junk_chars(self):
         self._run_check("<!DOCTYPE foo $ >", [('decl', 'DOCTYPE foo $ ')])
 
@@ -667,11 +479,15 @@ text
         self._run_check(html, expected)
 
     def test_broken_invalid_end_tag(self):
+        # This is technically wrong (the "> shouldn't be included in the 'data')
+        # but is probably not worth fixing it (in addition to all the cases of
+        # the previous test, it would require a full attribute parsing).
+        # see #13993
         html = '<b>This</b attr=">"> confuses the parser'
         expected = [('starttag', 'b', []),
                     ('data', 'This'),
                     ('endtag', 'b'),
-                    ('data', ' confuses the parser')]
+                    ('data', '"> confuses the parser')]
         self._run_check(html, expected)
 
     def test_correct_detection_of_start_tags(self):
@@ -721,64 +537,13 @@ text
         for html, expected in data:
             self._run_check(html, expected)
 
-    def test_eof_in_comments(self):
-        data = [
-            ('<!--', [('comment', '')]),
-            ('<!---', [('comment', '')]),
-            ('<!----', [('comment', '')]),
-            ('<!-----', [('comment', '-')]),
-            ('<!------', [('comment', '--')]),
-            ('<!----!', [('comment', '')]),
-            ('<!---!', [('comment', '-!')]),
-            ('<!---!>', [('comment', '-!>')]),
-            ('<!--foo', [('comment', 'foo')]),
-            ('<!--foo-', [('comment', 'foo')]),
-            ('<!--foo--', [('comment', 'foo')]),
-            ('<!--foo--!', [('comment', 'foo')]),
-            ('<!--<!--', [('comment', '<!')]),
-            ('<!--<!--!', [('comment', '<!')]),
-        ]
-        for html, expected in data:
-            self._run_check(html, expected)
-
-    def test_eof_in_declarations(self):
-        data = [
-            ('<!', [('comment', '')]),
-            ('<!-', [('comment', '-')]),
-            ('<![', [('comment', '[')]),
-            ('<!DOCTYPE', [('decl', 'DOCTYPE')]),
-            ('<!DOCTYPE ', [('decl', 'DOCTYPE ')]),
-            ('<!DOCTYPE html', [('decl', 'DOCTYPE html')]),
-            ('<!DOCTYPE html ', [('decl', 'DOCTYPE html ')]),
-            ('<!DOCTYPE html PUBLIC', [('decl', 'DOCTYPE html PUBLIC')]),
-            ('<!DOCTYPE html PUBLIC "foo', [('decl', 'DOCTYPE html PUBLIC "foo')]),
-            ('<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "foo',
-             [('decl', 'DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "foo')]),
-        ]
-        for html, expected in data:
-            self._run_check(html, expected)
-
-    @support.subTests('content', ['', 'x', 'x]', 'x]]'])
-    def test_eof_in_cdata(self, content):
-        self._run_check('<![CDATA[' + content,
-                        [('unknown decl', 'CDATA[' + content)])
-        self._run_check('<![CDATA[' + content,
-                        [('comment', '[CDATA[' + content)],
-                        collector=EventCollector(autocdata=True))
-        self._run_check('<svg><text y="100"><![CDATA[' + content,
-                        [('starttag', 'svg', []),
-                         ('starttag', 'text', [('y', '100')]),
-                         ('unknown decl', 'CDATA[' + content)])
-
-    def test_bogus_comments(self):
-        html = ('<!ELEMENT br EMPTY>'
-                '<! not really a comment >'
+    def test_broken_comments(self):
+        html = ('<! not really a comment >'
                 '<! not a comment either -->'
                 '<! -- close enough -->'
                 '<!><!<-- this was an empty comment>'
                 '<!!! another bogus comment !!!>')
         expected = [
-            ('comment', 'ELEMENT br EMPTY'),
             ('comment', ' not really a comment '),
             ('comment', ' not a comment either --'),
             ('comment', ' -- close enough --'),
@@ -819,57 +584,8 @@ text
             ('startendtag', 'img', [('src', 'mammoth.bmp')]),
             ('unknown decl', 'endif')
         ]
-
         self._run_check(html, expected)
 
-    @support.subTests('content', [
-        'just some plain text',
-        '<!-- not a comment -->',
-        '&not-an-entity-ref;',
-        "<not a='start tag'>",
-        '',
-        '[[I have many brackets]]',
-        'I have a > in the middle',
-        'I have a ]] in the middle',
-        '] ]>',
-        ']] >',
-        ('\n'
-         '    if (a < b && a > b) {\n'
-         '        printf("[<marquee>How?</marquee>]");\n'
-         '    }\n'),
-    ])
-    def test_cdata_section_content(self, content):
-        # See "13.2.5.42 Markup declaration open state",
-        # "13.2.5.69 CDATA section state", and issue bpo-32876.
-        html = f'<svg><text y="100"><![CDATA[{content}]]></text></svg>'
-        expected = [
-            ('starttag', 'svg', []),
-            ('starttag', 'text', [('y', '100')]),
-            ('unknown decl', 'CDATA[' + content),
-            ('endtag', 'text'),
-            ('endtag', 'svg'),
-        ]
-        self._run_check(html, expected)
-        self._run_check(html, expected, collector=EventCollector(autocdata=True))
-
-    def test_cdata_section(self):
-        # See "13.2.5.42 Markup declaration open state".
-        html = ('<![CDATA[foo<br>bar]]>'
-                '<svg><text y="100"><![CDATA[foo<br>bar]]></text></svg>'
-                '<![CDATA[foo<br>bar]]>')
-        expected = [
-            ('comment', '[CDATA[foo<br'),
-            ('data', 'bar]]>'),
-            ('starttag', 'svg', []),
-            ('starttag', 'text', [('y', '100')]),
-            ('unknown decl', 'CDATA[foo<br>bar'),
-            ('endtag', 'text'),
-            ('endtag', 'svg'),
-            ('comment', '[CDATA[foo<br'),
-            ('data', 'bar]]>'),
-        ]
-        self._run_check(html, expected, collector=EventCollector(autocdata=True))
-
     def test_convert_charrefs_dropped_text(self):
         # #23144: make sure that all the events are triggered when
         # convert_charrefs is True, even if we don't call .close()
@@ -882,26 +598,6 @@ text
              ('endtag', 'a'), ('data', ' bar & baz')]
         )
 
-    @support.requires_resource('cpu')
-    def test_eof_no_quadratic_complexity(self):
-        # Each of these examples used to take about an hour.
-        # Now they take a fraction of a second.
-        def check(source):
-            parser = html.parser.HTMLParser()
-            parser.feed(source)
-            parser.close()
-        n = 120_000
-        check("<a " * n)
-        check("<a a=" * n)
-        check("</a " * 14 * n)
-        check("</a a=" * 11 * n)
-        check("<!--" * 4 * n)
-        check("<!" * 60 * n)
-        check("<?" * 19 * n)
-        check("</$" * 15 * n)
-        check("<![CDATA[" * 9 * n)
-        check("<!doctype" * 35 * n)
-
 
 class AttributesTestCase(TestCaseBase):
 
@@ -910,15 +606,9 @@ class AttributesTestCase(TestCaseBase):
           ("starttag", "a", [("b", "v"), ("c", "v"), ("d", "v"), ("e", None)])
         ]
         self._run_check("""<a b='v' c="v" d=v e>""", output)
-        self._run_check("<a foo==bar>", [('starttag', 'a', [('foo', '=bar')])])
-        self._run_check("<a foo =bar>", [('starttag', 'a', [('foo', 'bar')])])
-        self._run_check("<a foo\t=bar>", [('starttag', 'a', [('foo', 'bar')])])
-        self._run_check("<a foo\v=bar>", [('starttag', 'a', [('foo\v', 'bar')])])
-        self._run_check("<a foo\xa0=bar>", [('starttag', 'a', [('foo\xa0', 'bar')])])
-        self._run_check("<a foo= bar>", [('starttag', 'a', [('foo', 'bar')])])
-        self._run_check("<a foo=\tbar>", [('starttag', 'a', [('foo', 'bar')])])
-        self._run_check("<a foo=\vbar>", [('starttag', 'a', [('foo', '\vbar')])])
-        self._run_check("<a foo=\xa0bar>", [('starttag', 'a', [('foo', '\xa0bar')])])
+        self._run_check("""<a  b = 'v' c = "v" d = v e>""", output)
+        self._run_check("""<a\nb\n=\n'v'\nc\n=\n"v"\nd\n=\nv\ne>""", output)
+        self._run_check("""<a\tb\t=\t'v'\tc\t=\t"v"\td\t=\tv\te>""", output)
 
     def test_attr_values(self):
         self._run_check("""<a b='xxx\n\txxx' c="yyy\t\nyyy" d='\txyz\n'>""",
@@ -927,10 +617,6 @@ class AttributesTestCase(TestCaseBase):
                                             ("d", "\txyz\n")])])
         self._run_check("""<a b='' c="">""",
                         [("starttag", "a", [("b", ""), ("c", "")])])
-        self._run_check("<a b=\tx c=\ny>",
-                        [('starttag', 'a', [('b', 'x'), ('c', 'y')])])
-        self._run_check("<a b=\v c=\xa0>",
-                        [("starttag", "a", [("b", "\v"), ("c", "\xa0")])])
         # Regression test for SF patch #669683.
         self._run_check("<e a=rgb(1,2,3)>",
                         [("starttag", "e", [("a", "rgb(1,2,3)")])])
@@ -997,17 +683,13 @@ class AttributesTestCase(TestCaseBase):
         )
         expected = [
             ('starttag', 'a', [('href', "test'style='color:red;bad1'")]),
-            ('data', 'test - bad1'),
-            ('endtag', 'a'),
+            ('data', 'test - bad1'), ('endtag', 'a'),
             ('starttag', 'a', [('href', "test'+style='color:red;ba2'")]),
-            ('data', 'test - bad2'),
-            ('endtag', 'a'),
+            ('data', 'test - bad2'), ('endtag', 'a'),
             ('starttag', 'a', [('href', "test'\xa0style='color:red;bad3'")]),
-            ('data', 'test - bad3'),
-            ('endtag', 'a'),
+            ('data', 'test - bad3'), ('endtag', 'a'),
             ('starttag', 'a', [('href', "test'\xa0style='color:red;bad4'")]),
-            ('data', 'test - bad4'),
-            ('endtag', 'a'),
+            ('data', 'test - bad4'), ('endtag', 'a')
         ]
         self._run_check(html, expected)
 
@@ -1105,6 +787,5 @@ class AttributesTestCase(TestCaseBase):
                             ('starttag', 'form',
                                 [('action', 'bogus|&#()value')])])
 
-
 if __name__ == "__main__":
     unittest.main()
